@@ -6,7 +6,7 @@ from numbers import Number
 from operator import getitem, add
 
 import numpy as np
-from toolz import merge, first, accumulate
+from toolz import merge, first, accumulate, pluck
 
 from ..base import tokenize
 from ..compatibility import long
@@ -182,6 +182,9 @@ def slice_wrap_lists(out_name, in_name, blockdims, index):
     """
     shape = tuple(map(sum, blockdims))
     assert all(isinstance(i, (slice, list, int, long)) for i in index)
+    assert len(blockdims) == len(index)
+    for bd, i in zip(blockdims, index):
+        check_index(i, sum(bd))
 
     # Change indices like -1 to 9
     index2 = posify_index(shape, index)
@@ -242,9 +245,10 @@ def slice_slices_and_integers(out_name, in_name, blockdims, index):
 
     # Get a list (for each dimension) of dicts{blocknum: slice()}
     block_slices = list(map(_slice_1d, shape, blockdims, index))
+    sorted_block_slices = [sorted(i.items()) for i in block_slices]
 
     # (in_name, 1, 1, 2), (in_name, 1, 1, 4), (in_name, 2, 1, 2), ...
-    in_names = list(product([in_name], *[sorted(i.keys()) for i in block_slices]))
+    in_names = list(product([in_name], *[pluck(0, s) for s in sorted_block_slices]))
 
     # (out_name, 0, 0, 0), (out_name, 0, 0, 1), (out_name, 0, 1, 0), ...
     out_names = list(product([out_name],
@@ -252,7 +256,7 @@ def slice_slices_and_integers(out_name, in_name, blockdims, index):
                                  for d, i in zip(block_slices, index)
                                  if not isinstance(i, (int, long))]))
 
-    all_slices = list(product(*[i.values() for i in block_slices]))
+    all_slices = list(product(*[pluck(1, s) for s in sorted_block_slices]))
 
     dsk_out = dict((out_name, (getitem, in_name, slices))
                    for out_name, in_name, slices
@@ -616,3 +620,49 @@ def replace_ellipsis(n, index):
     return (index[:loc]
           + (slice(None, None, None),) * (n - len(index) + 1)
           + index[loc+1:])
+
+
+def check_index(ind, dimension):
+    """ Check validity of index for a given dimension
+
+    Examples
+    --------
+    >>> check_index(3, 5)
+    >>> check_index(5, 5)
+    Traceback (most recent call last):
+    ...
+    IndexError: Index is not smaller than dimension 5 >= 5
+
+    >>> check_index(6, 5)
+    Traceback (most recent call last):
+    ...
+    IndexError: Index is not smaller than dimension 6 >= 5
+
+    >>> check_index(-1, 5)
+    >>> check_index(-6, 5)
+    Traceback (most recent call last):
+    ...
+    IndexError: Negative index is not greater than negative dimension -6 <= -5
+
+    >>> check_index([1, 2], 5)
+    >>> check_index([6, 3], 5)
+    Traceback (most recent call last):
+    ...
+    IndexError: Index is not smaller than dimension 6 >= 5
+
+    >>> check_index(slice(0, 3), 5)
+    """
+    if isinstance(ind, list):
+        for i in ind:
+            check_index(i, dimension)
+    elif isinstance(ind, slice):
+        return
+
+    elif ind >= dimension:
+        raise IndexError("Index is not smaller than dimension %d >= %d"
+                % (ind, dimension))
+
+    elif ind <= -dimension:
+        raise IndexError(
+                "Negative index is not greater than negative dimension %d <= -%d"
+                % (ind, dimension))
