@@ -15,7 +15,8 @@ from ..utils import ignoring
 
 from toolz import (merge, frequencies, merge_with, take, reduce,
                    join, reduceby, valmap, count, map, partition_all, filter,
-                   remove, pluck, groupby, topk, compose, drop, curry)
+                   remove, pluck, groupby, topk, compose, curry)
+from toolz.compatibility import iteritems, zip
 import toolz
 with ignoring(ImportError):
     from cytoolz import (frequencies, merge_with, join, reduceby,
@@ -23,7 +24,7 @@ with ignoring(ImportError):
 
 from ..base import Base, normalize_token, tokenize
 from ..compatibility import (apply, BytesIO, unicode, urlopen, urlparse,
-        GzipFile)
+                             GzipFile)
 from ..core import list2, quote, istask, get_dependencies, reverse_dict
 from ..multiprocessing import get as mpget
 from ..optimize import fuse, cull, inline
@@ -391,10 +392,7 @@ class Bag(Base):
 
         Bag.foldby
         """
-        token = tokenize(self, binop, combine, initial)
         combine = combine or binop
-        a = 'foldbinop-{0}-{1}'.format(funcname(binop), token)
-        b = 'foldcombine-{0}-{1}'.format(funcname(combine), token)
         initial = quote(initial)
         if initial is not no_default:
             return self.reduction(curry(_reduce, binop, initial=initial),
@@ -412,12 +410,11 @@ class Bag(Base):
         >>> dict(b.frequencies())  # doctest: +SKIP
         {'Alice': 2, 'Bob', 1}
         """
-        return self.reduction(compose(list, dictitems, frequencies),
-                              merge_frequencies,
+        return self.reduction(frequencies, merge_frequencies,
                               out_type=Bag, split_every=split_every,
-                              name='frequencies')
+                              name='frequencies').map_partitions(dictitems)
 
-    def topk(self, k, key=None):
+    def topk(self, k, key=None, split_every=None):
         """ K largest elements in collection
 
         Optionally ordered by some key function
@@ -429,19 +426,14 @@ class Bag(Base):
         >>> list(b.topk(2, lambda x: -x))  # doctest: +SKIP
         [3, 4]
         """
-        token = tokenize(self, k, key)
-        a = 'topk-a-' + token
-        b = 'topk-b-' + token
         if key:
             if callable(key) and takes_multiple_arguments(key):
                 key = partial(apply, key)
-            func = partial(topk, key=key)
+            func = partial(topk, k, key=key)
         else:
-            func = topk
-        dsk = dict(((a, i), (list, (func, k, (self.name, i))))
-                   for i in range(self.npartitions))
-        dsk2 = {(b, 0): (list, (func, k, (toolz.concat, list(dsk.keys()))))}
-        return type(self)(merge(self.dask, dsk, dsk2), b, 1)
+            func = partial(topk, k)
+        return self.reduction(func, compose(func, toolz.concat), out_type=Bag,
+                              split_every=split_every, name='topk')
 
     def distinct(self):
         """ Distinct elements of collection
@@ -453,7 +445,7 @@ class Bag(Base):
         ['Alice', 'Bob']
         """
         return self.reduction(set, curry(apply, set.union), out_type=Bag,
-                name='distinct')
+                              name='distinct')
 
     def reduction(self, perpartition, aggregate, split_every=None,
                   out_type=Item, name=None):
@@ -493,7 +485,8 @@ class Bag(Base):
         while k > 1:
             c = fmt + str(depth)
             dsk2 = dict(((c, i), (aggregate, [(b, j) for j in inds]))
-                 for i, inds in enumerate(partition_all(split_every, range(k))))
+                        for i, inds in enumerate(partition_all(split_every,
+                                                               range(k))))
             dsk.update(dsk2)
             k = len(dsk2)
             b = c
@@ -850,13 +843,13 @@ class Bag(Base):
               "Repartition only supports going to fewer partitions\n"
               " old: %d  new: %d" % (self.npartitions, npartitions))
         size = self.npartitions / npartitions
-        L = [int(i * self.npartitions / npartitions)
-                for i in range(npartitions + 1)]
+        L = [int(i * size) for i in range(npartitions + 1)]
         name = 'repartition-%d-%s' % (npartitions, self.name)
         dsk = dict(((name, i), (list,
-                                (toolz.concat, [(self.name, j)
-                                            for j in range(L[i], L[i + 1])])))
-                    for i in range(npartitions))
+                                (toolz.concat,
+                                 [(self.name, j) for j in range(L[i], L[i + 1])]
+                                 )))
+                   for i in range(npartitions))
         return Bag(merge(self.dask, dsk), name, npartitions)
 
 
@@ -1127,7 +1120,7 @@ def from_castra(x, columns=None, index=False):
     name = 'from-castra-' + tokenize(os.path.getmtime(x.path), x.path,
                                      columns, index)
     dsk = dict(((name, i), (load_castra_partition, x, part, columns, index))
-                for i, part in enumerate(x.partitions))
+               for i, part in enumerate(x.partitions))
     return Bag(dsk, name, len(x.partitions))
 
 
@@ -1144,8 +1137,7 @@ def load_castra_partition(castra, part, columns, index):
         items = df.iteritems() if index else iter(df)
 
     items = list(items)
-    if (items and isinstance(items[0], tuple)
-              and type(items[0]) is not tuple):
+    if items and isinstance(items[0], tuple) and type(items[0]) is not tuple:
         names = items[0]._fields
         items = [dict(zip(names, item)) for item in items]
 
@@ -1311,7 +1303,15 @@ def from_imperative(values):
 
 
 def merge_frequencies(seqs):
-    return list(merge_with(sum, map(dict, seqs)).items())
+    first, rest = seqs[0], seqs[1:]
+    if not rest:
+        return first
+    out = defaultdict(int)
+    out.update(first)
+    for d in rest:
+        for k, v in iteritems(d):
+            out[k] += v
+    return out
 
 
 def bag_range(n, npartitions):
@@ -1329,7 +1329,7 @@ def bag_range(n, npartitions):
     name = 'range-%d-npartitions-%d' % (n, npartitions)
     ijs = list(enumerate(take(npartitions, range(0, n, size))))
     dsk = dict(((name, i), (reify, (range, j, min(j + size, n))))
-                for i, j in ijs)
+               for i, j in ijs)
 
     if n % npartitions != 0:
         i, j = ijs[-1]
@@ -1338,6 +1338,54 @@ def bag_range(n, npartitions):
     return Bag(dsk, name, npartitions)
 
 
+def bag_zip(*bags):
+    """ Partition-wise bag zip
+
+    All passed bags must have the same number of partitions.
+
+    NOTE: corresponding partitions should have the same length; if they do not,
+    the "extra" elements from the longer partition(s) will be dropped.  If you
+    have this case chances are that what you really need is a data alignment
+    mechanism like pandas's, and not a missing value filler like zip_longest.
+
+    Examples
+    --------
+
+    Correct usage:
+
+    >>> import dask.bag as db
+    >>> evens = db.from_sequence(range(0, 10, 2), partition_size=4)
+    >>> odds = db.from_sequence(range(1, 10, 2), partition_size=4)
+    >>> pairs = db.zip(evens, odds)
+    >>> list(pairs)
+    [(0, 1), (2, 3), (4, 5), (6, 7), (8, 9)]
+
+    Incorrect usage:
+
+    >>> numbers = db.range(20) # doctest: +SKIP
+    >>> fizz = numbers.filter(lambda n: n % 3 == 0) # doctest: +SKIP
+    >>> buzz = numbers.filter(lambda n: n % 5 == 0) # doctest: +SKIP
+    >>> fizzbuzz = db.zip(fizz, buzz) # doctest: +SKIP
+    >>> list(fizzbuzzz) # doctest: +SKIP
+    [(0, 0), (3, 5), (6, 10), (9, 15), (12, 20), (15, 25), (18, 30)]
+
+    When what you really wanted was more along the lines of:
+    >>> list(fizzbuzzz) # doctest: +SKIP
+    [(0, 0), (3, None), (None, 5), (6, None), (None 10), (9, None),
+     (12, None), (15, 15), (18, None), (None, 20), (None, 25), (None, 30)]
+    """
+    npartitions = bags[0].npartitions
+    assert all(bag.npartitions == npartitions for bag in bags)
+    # TODO: do more checks
+
+    name = 'zip-' + tokenize(*bags)
+    dsk = dict(
+        ((name, i), (reify, (zip,) + tuple((bag.name, i) for bag in bags)))
+        for i in range(npartitions))
+    bags_dsk = merge(*(bag.dask for bag in bags))
+    return Bag(merge(bags_dsk, dsk), name, npartitions)
+
+
 def _reduce(binop, sequence, initial=no_default):
     if initial is not no_default:
         return reduce(binop, sequence, initial)
