@@ -189,8 +189,11 @@ def test_frequencies():
 
 def test_topk():
     assert list(b.topk(4)) == [4, 4, 4, 3]
-    assert list(b.topk(4, key=lambda x: -x).compute(get=dask.get)) == \
-            [0, 0, 0, 1]
+    c = b.topk(4, key=lambda x: -x)
+    assert list(c) == [0, 0, 0, 1]
+    c2 = b.topk(4, key=lambda x: -x, split_every=2)
+    assert list(c2) == [0, 0, 0, 1]
+    assert c.name != c2.name
     assert b.topk(4).name == b.topk(4).name
 
 
@@ -831,6 +834,15 @@ def test_range():
         assert list(b) == list(range(100))
 
 
+@pytest.mark.parametrize("npartitions", [1, 7, 10, 28])
+def test_zip(npartitions, hi=1000):
+    evens = db.from_sequence(range(0, hi, 2), npartitions=npartitions)
+    odds = db.from_sequence(range(1, hi, 2), npartitions=npartitions)
+    pairs = db.zip(evens, odds)
+    assert pairs.npartitions == npartitions
+    assert list(pairs) == list(zip(range(0, hi, 2), range(1, hi, 2)))
+
+
 def test_repartition():
     for x, y in [(10, 5), (7, 3), (5, 1), (5, 4)]:
         b = db.from_sequence(range(20), npartitions=x)
