@@ -4,7 +4,7 @@ from .io import (read_csv, from_array, from_bcolz, from_array, from_bcolz,
                  from_pandas, from_dask_array, from_castra, read_hdf,
                  from_imperative)
 from .optimize import optimize
-from .multi import merge, concat
+from .multi import merge, concat, melt
 from .rolling import (rolling_count, rolling_sum, rolling_mean, rolling_median,
                       rolling_min, rolling_max, rolling_std, rolling_var,
                       rolling_skew, rolling_kurt, rolling_quantile, rolling_apply,
