@@ -28,6 +28,7 @@ from ..compatibility import unicode, apply, operator_div, bind_method
 from ..utils import (repr_long_list, IndexCallable,
                      pseudorandom, derived_from, different_seeds)
 from ..base import Base, compute, tokenize, normalize_token
+from ..async import get_sync
 
 no_default = '__no_default__'
 return_scalar = '__return_scalar__'
@@ -525,15 +526,15 @@ class _Frame(Base):
 
     @derived_from(pd.DataFrame)
     def to_hdf(self, path_or_buf, key, mode='a', append=False, complevel=0,
-               complib=None, fletcher32=False, **kwargs):
+               complib=None, fletcher32=False, get=get_sync, **kwargs):
         from .io import to_hdf
         return to_hdf(self, path_or_buf, key, mode, append, complevel, complib,
-                fletcher32, **kwargs)
+                fletcher32, get=get, **kwargs)
 
     @derived_from(pd.DataFrame)
-    def to_csv(self, filename, **kwargs):
+    def to_csv(self, filename, get=get_sync, **kwargs):
         from .io import to_csv
-        return to_csv(self, filename, **kwargs)
+        return to_csv(self, filename, get=get, **kwargs)
 
     def to_imperative(self):
         """ Convert dataframe into dask Values
@@ -1100,7 +1101,7 @@ class Series(_Frame):
 
     @derived_from(pd.Series)
     def isin(self, other):
-        return elemwise(pd.Series.isin, self, other)
+        return elemwise(pd.Series.isin, self, list(other))
 
     @derived_from(pd.Series)
     def map(self, arg, na_action=None):
@@ -1149,7 +1150,7 @@ class Series(_Frame):
     @derived_from(pd.Series)
     def to_frame(self, name=None):
         _name = name if name is not None else self.name
-        return map_partitions(pd.Series.to_frame, [_name], self, name)
+        return map_partitions(pd.Series.to_frame, self._pd.to_frame(name), self, name)
 
     @classmethod
     def _bind_operator_method(cls, name, op):
@@ -1164,7 +1165,35 @@ class Series(_Frame):
         bind_method(cls, name, meth)
 
     def apply(self, func, convert_dtype=True, name=no_default, args=(), **kwds):
-        """ Parallel version of pandas.Series.apply """
+        """ Parallel version of pandas.Series.apply
+
+        This mimics the pandas version except for the following:
+
+        1.  The user should provide output name.
+
+        Parameters
+        ----------
+
+        func: function
+            Function to apply
+        convert_dtype: boolean, default True
+            Try to find better dtype for elementwise function results.
+            If False, leave as dtype=object
+        name: list, scalar or None, optional
+            If list is given, the result is a DataFrame which columns is
+            specified list. Otherwise, the result is a Series which name is
+            given scalar or None (no name). If name keyword is not given, dask
+            tries to infer the result type using its beggining of data. This
+            inference may take some time and lead to unexpected result.
+        args: tuple
+            Positional arguments to pass to function in addition to the array/series
+
+        Additional keyword arguments will be passed as keywords to the function
+
+        Returns
+        -------
+        applied : Series or DataFrame depending on name keyword
+        """
 
         if name is no_default:
             msg = ("name is not specified, inferred from partial data. "
@@ -1181,6 +1210,25 @@ class Series(_Frame):
         return map_partitions(pd.Series.apply, name, self, func,
                               convert_dtype, args, **kwds)
 
+    @derived_from(pd.Series)
+    def cov(self, other, min_periods=None):
+        from .multi import concat
+        if not isinstance(other, Series):
+            raise TypeError("other must be a dask.dataframe.Series")
+        df = concat([self, other], axis=1)
+        return cov_corr(df, min_periods, scalar=True)
+
+    @derived_from(pd.Series)
+    def corr(self, other, method='pearson', min_periods=None):
+        from .multi import concat
+        if not isinstance(other, Series):
+            raise TypeError("other must be a dask.dataframe.Series")
+        if method != 'pearson':
+            raise NotImplementedError("Only Pearson correlation has been "
+                                      "implemented")
+        df = concat([self, other], axis=1)
+        return cov_corr(df, min_periods, corr=True, scalar=True)
+
 
 class Index(Series):
 
@@ -1334,9 +1382,9 @@ class DataFrame(_Frame):
             return self._pd_cache.dtypes
 
     @derived_from(pd.DataFrame)
-    def set_index(self, other, **kwargs):
+    def set_index(self, other, drop=True, **kwargs):
         from .shuffle import set_index
-        return set_index(self, other, **kwargs)
+        return set_index(self, other, drop=drop, **kwargs)
 
     def set_partition(self, column, divisions, **kwargs):
         """ Set explicit divisions for new column index
@@ -1428,7 +1476,7 @@ class DataFrame(_Frame):
         return self.map_partitions(f, how=how, subset=subset)
 
     def to_castra(self, fn=None, categories=None, sorted_index_column=None,
-                  compute=True):
+                  compute=True, get=get_sync):
         """ Write DataFrame to Castra on-disk store
 
         See https://github.com/blosc/castra for details
@@ -1439,7 +1487,7 @@ class DataFrame(_Frame):
         """
         from .io import to_castra
         return to_castra(self, fn, categories, sorted_index_column,
-                         compute=compute)
+                         compute=compute, get=get)
 
     def to_bag(self, index=False):
         """Convert to a dask Bag of tuples of each row.
@@ -1572,9 +1620,35 @@ class DataFrame(_Frame):
 
         This mimics the pandas version except for the following:
 
-        1.  The user must specify axis=0 explicitly
-        2.  The user must provide output columns or column
+        1.  The user must specify axis=1 explicitly.
+        2.  The user should provide output columns.
+
+        Parameters
+        ----------
+
+        func: function
+            Function to apply to each column
+        axis: {0 or 'index', 1 or 'columns'}, default 0
+            - 0 or 'index': apply function to each column (NOT SUPPORTED)
+            - 1 or 'columns': apply function to each row
+        columns: list, scalar or None
+            If list is given, the result is a DataFrame which columns is
+            specified list. Otherwise, the result is a Series which name is
+            given scalar or None (no name). If name keyword is not given, dask
+            tries to infer the result type using its beggining of data. This
+            inference may take some time and lead to unexpected result
+        args : tuple
+            Positional arguments to pass to function in addition to the array/series
+
+        Additional keyword arguments will be passed as keywords to the function
+
+        Returns
+        -------
+        applied : Series or DataFrame depending on name keyword
         """
+
+        axis = self._validate_axis(axis)
+
         if axis == 0:
             raise NotImplementedError(
                     "dd.DataFrame.apply only supports axis=1\n"
@@ -1594,6 +1668,16 @@ class DataFrame(_Frame):
         return map_partitions(pd.DataFrame.apply, columns, self, func, axis,
                               False, False, None, args, **kwds)
 
+    @derived_from(pd.DataFrame)
+    def cov(self, min_periods=None):
+        return cov_corr(self, min_periods)
+
+    @derived_from(pd.DataFrame)
+    def corr(self, method='pearson', min_periods=None):
+        if method != 'pearson':
+            raise NotImplementedError("Only Pearson correlation has been "
+                                      "implemented")
+        return cov_corr(self, min_periods, True)
 
 
 # bind operators
@@ -1687,8 +1771,7 @@ def _loc(df, start, stop, include_right_boundary=True):
     """
     result = df.loc[start:stop]
     if not include_right_boundary:
-        right_index = result.index.get_slice_bound(stop, 'left',
-                                                   result.index.inferred_type)
+        right_index = result.index.get_slice_bound(stop, 'left', 'loc')
         result = result.iloc[:right_index]
     return result
 
@@ -1813,7 +1896,6 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
 
     Parameters
     ----------
-
     args: dask.DataFrames
         All Dataframes should be partitioned and indexed equivalently
     chunk: function [block-per-arg] -> block
@@ -1821,6 +1903,8 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
     aggregate: function concatenated-block -> block
         Function to operate on the concatenated result of chunk
 
+    Examples
+    --------
     >>> def chunk(a_block, b_block):
     ...     pass
 
@@ -1839,9 +1923,13 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
     token = token or 'apply-concat-apply'
 
     a = '{0}--first-{1}'.format(token, token_key)
-    dsk = dict(((a, i), (apply, chunk, (list, [(x._name, i)
-                                               if isinstance(x, _Frame)
-                                               else x for x in args])))
+    if len(args) == 1 and isinstance(args[0], _Frame):
+        dsk = dict(((a, i), (chunk, key))
+                   for i, key in enumerate(args[0]._keys()))
+    else:
+        dsk = dict(((a, i), (apply, chunk, [(x._name, i)
+                                            if isinstance(x, _Frame)
+                                            else x for x in args]))
                for i in range(args[0].npartitions))
 
     b = '{0}--second-{1}'.format(token, token_key)
@@ -2092,6 +2180,99 @@ def quantile(df, q):
     return return_type(dsk, name3, df.name, new_divisions)
 
 
+def cov_corr(df, min_periods=None, corr=False, scalar=False):
+    """DataFrame covariance and pearson correlation.
+
+    Computes pairwise covariance or correlation of columns, excluding NA/null
+    values.
+
+    Parameters
+    ----------
+    df : DataFrame
+    min_periods : int, optional
+        Minimum number of observations required per pair of columns
+        to have a valid result.
+    corr : bool, optional
+        If True, compute the Pearson correlation. If False [default], compute
+        the covariance.
+    scalar : bool, optional
+        If True, compute covariance between two variables as a scalar. Only
+        valid if `df` has 2 columns.  If False [default], compute the entire
+        covariance/correlation matrix.
+    """
+    if min_periods is None:
+        min_periods = 2
+    elif min_periods < 2:
+        raise ValueError("min_periods must be >= 2")
+    prefix = 'corr' if corr else 'cov'
+    df = df._get_numeric_data()
+    name = '{0}-agg-{1}'.format(prefix, tokenize(df, min_periods, scalar))
+    if scalar and len(df.columns) != 2:
+        raise ValueError("scalar only valid for 2 column dataframe")
+    k = '{0}-chunk-{1}'.format(prefix, df._name)
+    dsk = dict(((k, i), (cov_corr_chunk, f, corr))
+               for (i, f) in enumerate(df._keys()))
+    dsk[(name, 0)] = (cov_corr_agg, list(dsk.keys()), df._pd, min_periods,
+                      corr, scalar)
+    dsk = merge(df.dask, dsk)
+    if scalar:
+        return Scalar(dsk, name)
+    return DataFrame(dsk, name, df._pd, (df.columns[0], df.columns[-1]))
+
+
+def cov_corr_chunk(df, corr=False):
+    """Chunk part of a covariance or correlation computation"""
+    mat = df.values
+    mask = np.isfinite(mat)
+    keep = np.bitwise_and(mask[:, None, :], mask[:, :, None])
+
+    x = np.where(keep, mat[:, None, :], np.nan)
+    sums = np.nansum(x, 0)
+    counts = keep.astype('int').sum(0)
+    cov = df.cov().values
+    dtype = [('sum', sums.dtype), ('count', counts.dtype), ('cov', cov.dtype)]
+    if corr:
+        m = np.nansum((x - sums/np.where(counts, counts, np.nan))**2, 0)
+        dtype.append(('m', m.dtype))
+
+    out = np.empty(counts.shape, dtype=dtype)
+    out['sum'] = sums
+    out['count'] = counts
+    out['cov'] = cov * (counts - 1)
+    if corr:
+        out['m'] = m
+    return out
+
+
+def cov_corr_agg(data, meta, min_periods=2, corr=False, scalar=False):
+    """Aggregation part of a covariance or correlation computation"""
+    data = np.concatenate(data).reshape((len(data),) + data[0].shape)
+    sums = np.nan_to_num(data['sum'])
+    counts = data['count']
+
+    totals = np.cumsum(sums, 0)
+    ns = np.cumsum(counts, 0)
+
+    ts = ns[:-1] * sums[1:] - counts[1:] * totals[:-1]
+    dens = ns[1:] * ns[:-1] * counts[1:]
+    dens = np.where(dens, dens, np.nan)
+    C = (np.nansum(data['cov'], 0) +
+         np.nansum(ts * ts.transpose((0, 2, 1)) / dens, 0))
+    C[ns[-1] < min_periods] = np.nan
+    nobs = np.where(ns[-1], ns[-1], np.nan)
+    if corr:
+        mu = totals[-1] / nobs
+        counts_na = np.where(counts, counts, np.nan)
+        m2 = np.nansum(data['m'] + counts*(sums/counts_na - mu)**2, axis=0)
+        den = np.sqrt(m2 * m2.T)
+    else:
+        den = nobs - 1
+    mat = C/den
+    if scalar:
+        return mat[0, 1]
+    return pd.DataFrame(mat, columns=meta.columns, index=meta.columns)
+
+
 def pd_split(df, p, random_state=None):
     """ Split DataFrame into multiple pieces pseudorandomly
 
