@@ -327,7 +327,7 @@ def from_array(x, chunksize=50000, columns=None):
     return _Frame(dsk, name, dummy, divisions)
 
 
-def from_pandas(data, npartitions, sort=True):
+def from_pandas(data, npartitions=None, chunksize=None, sort=True):
     """Construct a dask object from a pandas object.
 
     If given a ``pandas.Series`` a ``dask.Series`` will be returned. If given a
@@ -338,8 +338,11 @@ def from_pandas(data, npartitions, sort=True):
     ----------
     df : pandas.DataFrame or pandas.Series
         The DataFrame/Series with which to construct a dask DataFrame/Series
-    npartitions : int
-        The number of partitions of the index to create
+    npartitions : int, optional
+        The number of partitions of the index to create.
+    chunksize : int, optional
+        The size of the partitions of the index.
+
 
     Returns
     -------
@@ -381,8 +384,14 @@ def from_pandas(data, npartitions, sort=True):
     if not isinstance(data, (pd.Series, pd.DataFrame)):
         raise TypeError("Input must be a pandas DataFrame or Series")
 
+    if ((npartitions is None) == (chunksize is None)):
+        raise ValueError('Exactly one of npartitions and chunksize must be specified.')
     nrows = len(data)
-    chunksize = int(ceil(nrows / npartitions))
+    if chunksize is None:
+        chunksize = int(ceil(nrows / npartitions))
+    else:
+        npartitions = int(ceil(nrows / chunksize))
+
     if sort and not data.index.is_monotonic_increasing:
         data = data.sort_index(ascending=True)
     if sort:
@@ -407,12 +416,12 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, **kwargs):
 
     x : bcolz.ctable
         Input data
-    chunksize : int (optional)
+    chunksize : int, optional
         The size of blocks to pull out from ctable.  Ideally as large as can
         comfortably fit in memory
-    categorize : bool (defaults to True)
+    categorize : bool, defaults to True
         Automatically categorize all string dtypes
-    index : string (optional)
+    index : string, optional
         Column to make the index
 
     See Also
@@ -439,9 +448,8 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, **kwargs):
                 categories[name] = da.unique(a)
 
     columns = tuple(x.dtype.names)
-    divisions = (0,) + tuple(range(-1, len(x), chunksize))[1:]
-    if divisions[-1] != len(x) - 1:
-        divisions = divisions + (len(x) - 1,)
+    divisions = tuple(range(0, len(x), chunksize))
+    divisions = divisions + (len(x) - 1,)
     if x.rootdir:
         token = tokenize((x.rootdir, os.path.getmtime(x.rootdir)), chunksize,
                          categorize, index, kwargs)
@@ -483,17 +491,17 @@ def dataframe_from_ctable(x, slc, columns=None, categories=None):
     >>> x = bcolz.ctable([[1, 2, 3, 4], [10, 20, 30, 40]], names=['a', 'b'])
     >>> dataframe_from_ctable(x, slice(1, 3))
        a   b
-    0  2  20
-    1  3  30
+    1  2  20
+    2  3  30
 
     >>> dataframe_from_ctable(x, slice(1, 3), columns=['b'])
         b
-    0  20
-    1  30
+    1  20
+    2  30
 
     >>> dataframe_from_ctable(x, slice(1, 3), columns='b')
-    0    20
-    1    30
+    1    20
+    2    30
     Name: b, dtype: int...
 
     """
@@ -504,6 +512,13 @@ def dataframe_from_ctable(x, slc, columns=None, categories=None):
         columns = list(columns)
 
     x = x[columns]
+    if type(slc) is slice:
+        start = slc.start
+        stop = slc.stop if slc.stop < len(x) else len(x)
+    else:
+        start = slc[0].start
+        stop = slc[0].stop if slc[0].stop < len(x) else len(x)
+    idx = pd.Index(range(start, stop))
 
     if isinstance(x, bcolz.ctable):
         chunks = [x[name][slc] for name in columns]
@@ -513,7 +528,8 @@ def dataframe_from_ctable(x, slc, columns=None, categories=None):
                                                 categories[name], True)
                        if name in categories else chunk
                        for name, chunk in zip(columns, chunks)]
-        return pd.DataFrame(dict(zip(columns, chunks)), columns=columns)
+        return pd.DataFrame(dict(zip(columns, chunks)), columns=columns,
+                            index=idx)
 
     elif isinstance(x, bcolz.carray):
         chunk = x[slc]
@@ -521,7 +537,7 @@ def dataframe_from_ctable(x, slc, columns=None, categories=None):
             chunk = pd.Categorical.from_codes(
                         np.searchsorted(categories[columns], chunk),
                         categories[columns], True)
-        return pd.Series(chunk, name=columns)
+        return pd.Series(chunk, name=columns, index=idx)
 
 
 def locked_df_from_ctable(*args, **kwargs):
@@ -610,7 +626,8 @@ def _link(token, result):
 
 @wraps(pd.DataFrame.to_hdf)
 def to_hdf(df, path_or_buf, key, mode='a', append=False, complevel=0,
-           complib=None, fletcher32=False, get=get_sync, **kwargs):
+           complib=None, fletcher32=False, get=get_sync, dask_kwargs=None,
+           **kwargs):
     name = 'to-hdf-' + uuid.uuid1().hex
 
     pd_to_hdf = getattr(df._partition_type, 'to_hdf')
@@ -619,19 +636,23 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, complevel=0,
     dsk[(name, 0)] = (_link, None,
                       (apply, pd_to_hdf,
                           (tuple, [(df._name, 0), path_or_buf, key]),
-                          {'mode':  mode, 'format': 'table', 'append': append,
-                           'complevel': complevel, 'complib': complib,
-                           'fletcher32': fletcher32}))
+                          merge(kwargs,
+                            {'mode':  mode, 'format': 'table', 'append': append,
+                             'complevel': complevel, 'complib': complib,
+                             'fletcher32': fletcher32})))
     for i in range(1, df.npartitions):
         dsk[(name, i)] = (_link, (name, i - 1),
                           (apply, pd_to_hdf,
                            (tuple, [(df._name, i), path_or_buf, key]),
-                           {'mode': 'a', 'format': 'table', 'append': True,
-                            'complevel': complevel, 'complib': complib,
-                            'fletcher32': fletcher32}))
+                           merge(kwargs,
+                             {'mode': 'a', 'format': 'table', 'append': True,
+                              'complevel': complevel, 'complib': complib,
+                              'fletcher32': fletcher32})))
+
+    dask_kwargs = dask_kwargs or {}
 
     DataFrame._get(merge(df.dask, dsk), (name, df.npartitions - 1),
-                   get=get_sync, **kwargs)
+                   get=get, **dask_kwargs)
 
 
 dont_use_fixed_error_message = """
@@ -679,21 +700,22 @@ def _read_single_hdf(path, key, start=0, stop=None, columns=None,
         Get the data frame corresponding to one path and one key (which should
         not contain any wildcards).
         """
-        if columns is None:
-            columns = list(pd.read_hdf(path, key, stop=0).columns)
+        empty = pd.read_hdf(path, key, stop=0)
+        if columns is not None:
+            empty = empty[columns]
 
         token = tokenize((path, os.path.getmtime(path), key, start,
-                          stop, columns, chunksize))
+                          stop, empty, chunksize))
         name = 'read-hdf-' + token
 
         dsk = dict(((name, i), (_pd_read_hdf, path, key, lock,
                                  {'start': s,
                                   'stop': s + chunksize,
-                                  'columns': columns}))
+                                  'columns': empty.columns}))
                     for i, s in enumerate(range(start, stop, chunksize)))
 
         divisions = [None] * (len(dsk) + 1)
-        return DataFrame(dsk, name, columns, divisions)
+        return DataFrame(dsk, name, empty, divisions)
 
     if lock is True:
         lock = Lock()
@@ -767,7 +789,7 @@ def read_hdf(pattern, key, start=0, stop=None, columns=None,
 
 
 def to_castra(df, fn=None, categories=None, sorted_index_column=None,
-              compute=True):
+              compute=True, get=get_sync):
     """ Write DataFrame to Castra on-disk store
 
     See https://github.com/blosc/castra for details
@@ -797,17 +819,15 @@ def to_castra(df, fn=None, categories=None, sorted_index_column=None,
     dsk = merge(dsk, df.dask)
     keys = [(name, -1), (name, df.npartitions - 1)]
     if compute:
-        c, _ = DataFrame._get(dsk, keys, get=get_sync)
+        c, _ = DataFrame._get(dsk, keys, get=get)
         return c
     else:
         return dsk, keys
 
 
-def to_csv(df, filename, compression=None, **kwargs):
+def to_csv(df, filename, compression=None, get=None, **kwargs):
     if compression:
         raise NotImplementedError("Writing compressed csv files not supported")
-
-    myget = kwargs.pop('get', None)
     name = 'to-csv-' + uuid.uuid1().hex
 
     dsk = dict()
@@ -823,7 +843,7 @@ def to_csv(df, filename, compression=None, **kwargs):
                            (lambda df, fn, kwargs: df.to_csv(fn, **kwargs),
                              (df._name, i), filename, kwargs2))
 
-    DataFrame._get(merge(dsk, df.dask), (name, df.npartitions - 1), get=myget)
+    DataFrame._get(merge(dsk, df.dask), (name, df.npartitions - 1), get=get)
 
 
 def to_bag(df, index=False):
@@ -840,7 +860,7 @@ def to_bag(df, index=False):
     return Bag(dsk, name, df.npartitions)
 
 
-def from_imperative(dfs, columns, divisions=None):
+def from_imperative(dfs, metadata=None, divisions=None, columns=None):
     """ Create DataFrame from many imperative objects
 
     Parameters
@@ -848,11 +868,12 @@ def from_imperative(dfs, columns, divisions=None):
     dfs: list of Values
         An iterable of dask.imperative.Value objects, such as come from dask.do
         These comprise the individual partitions of the resulting dataframe
-    columns: list or string
-        The list of column names if the result is a DataFrame
-        Or the single column name if the result is a Series
+    metadata: list or string of column names or empty dataframe
     divisions: list or None
     """
+    if columns is not None:
+        print("Deprecation warning: Use metadata argument, not columns")
+        metadata = columns
     from dask.imperative import Value
     if isinstance(dfs, Value):
         dfs = [dfs]
@@ -866,7 +887,7 @@ def from_imperative(dfs, columns, divisions=None):
     if divisions is None:
         divisions = [None] * (len(dfs) + 1)
 
-    if isinstance(columns, str):
-        return Series(merge(dsk, dsk2), name, columns, divisions)
+    if isinstance(metadata, str):
+        return Series(merge(dsk, dsk2), name, metadata, divisions)
     else:
-        return DataFrame(merge(dsk, dsk2), name, columns, divisions)
+        return DataFrame(merge(dsk, dsk2), name, metadata, divisions)
