@@ -137,13 +137,47 @@ def test_set_index():
     assert d4.index.name == 'b'
     assert eq(d4, full.set_index('b'))
 
+@pytest.mark.parametrize('drop', [True, False])
+def test_set_index_drop(drop):
+
+    pdf = pd.DataFrame({'A': list('ABAABBABAA'),
+                        'B': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
+                        'C': [1, 2, 3, 2, 1, 3, 2, 4, 2, 3]})
+    ddf = dd.from_pandas(pdf, 3)
+
+    assert eq(ddf.set_index('A', drop=drop),
+              pdf.set_index('A', drop=drop))
+    assert eq(ddf.set_index('B', drop=drop),
+              pdf.set_index('B', drop=drop))
+    assert eq(ddf.set_index('C', drop=drop),
+              pdf.set_index('C', drop=drop))
+    assert eq(ddf.set_index(ddf.A, drop=drop),
+              pdf.set_index(pdf.A, drop=drop))
+    assert eq(ddf.set_index(ddf.B, drop=drop),
+              pdf.set_index(pdf.B, drop=drop))
+    assert eq(ddf.set_index(ddf.C, drop=drop),
+              pdf.set_index(pdf.C, drop=drop))
+
+    # numeric columns
+    pdf = pd.DataFrame({0: list('ABAABBABAA'),
+                        1: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
+                        2: [1, 2, 3, 2, 1, 3, 2, 4, 2, 3]})
+    ddf = dd.from_pandas(pdf, 3)
+    assert eq(ddf.set_index(0, drop=drop),
+              pdf.set_index(0, drop=drop))
+    assert eq(ddf.set_index(2, drop=drop),
+              pdf.set_index(2, drop=drop))
+
 
 def test_set_index_raises_error_on_bad_input():
     df = pd.DataFrame({'a': [1, 2, 3, 4, 5, 6, 7],
                         'b': [7, 6, 5, 4, 3, 2, 1]})
     ddf = dd.from_pandas(df, 2)
 
-    assert raises(NotImplementedError, lambda: ddf.set_index(['a', 'b']))
+    msg = r"Dask dataframe does not yet support multi-indexes"
+    with tm.assertRaisesRegexp(NotImplementedError, msg):
+        ddf.set_index(['a', 'b'])
+
 
 def test_rename_columns():
     # GH 819
@@ -499,6 +533,8 @@ def test_unique():
 
 def test_isin():
     assert eq(d.a.isin([0, 1, 2]), full.a.isin([0, 1, 2]))
+    assert eq(d.a.isin(pd.Series([0, 1, 2])),
+              full.a.isin(pd.Series([0, 1, 2])))
 
 
 def test_len():
@@ -1336,6 +1372,7 @@ def test_to_frame():
     s = pd.Series([1, 2, 3], name='foo')
     a = dd.from_pandas(s, npartitions=2)
 
+    assert a.to_frame()._known_dtype
     assert eq(s.to_frame(), a.to_frame())
     assert eq(s.to_frame('bar'), a.to_frame('bar'))
 
@@ -1350,6 +1387,8 @@ def test_apply():
     # specify columns
     assert eq(ddf.apply(lambda xy: xy[0] + xy[1], axis=1, columns=None),
               df.apply(lambda xy: xy[0] + xy[1], axis=1))
+    assert eq(ddf.apply(lambda xy: xy[0] + xy[1], axis='columns', columns=None),
+              df.apply(lambda xy: xy[0] + xy[1], axis='columns'))
 
     # inferrence
     assert eq(ddf.apply(lambda xy: xy[0] + xy[1], axis=1),
@@ -1357,7 +1396,7 @@ def test_apply():
     assert eq(ddf.apply(lambda xy: xy, axis=1),
               df.apply(lambda xy: xy, axis=1))
 
-    # dataframe
+    # result will be dataframe
     func = lambda x: pd.Series([x, x])
     assert eq(ddf.x.apply(func, name=[0, 1]), df.x.apply(func))
     # inference
@@ -1367,6 +1406,52 @@ def test_apply():
     with tm.assertRaises(NotImplementedError):
         ddf.apply(lambda xy: xy, axis=0)
 
+    with tm.assertRaises(NotImplementedError):
+        ddf.apply(lambda xy: xy, axis='index')
+
+
+def test_cov():
+    df = pd.util.testing.makeMissingDataframe(0.3, 42)
+    ddf = dd.from_pandas(df, npartitions=3)
+
+    assert eq(ddf.cov(), df.cov())
+    assert eq(ddf.cov(10), df.cov(10))
+    assert ddf.cov()._name == ddf.cov()._name
+    assert ddf.cov(10)._name != ddf.cov()._name
+
+    a = df.A
+    b = df.B
+    da = dd.from_pandas(a, npartitions=3)
+    db = dd.from_pandas(b, npartitions=4)
+    assert eq(da.cov(db), a.cov(b))
+    assert eq(da.cov(db, 10), a.cov(b, 10))
+    assert da.cov(db)._name == da.cov(db)._name
+    assert da.cov(db, 10)._name != da.cov(db)._name
+
+
+def test_corr():
+    df = pd.util.testing.makeMissingDataframe(0.3, 42)
+    ddf = dd.from_pandas(df, npartitions=3)
+
+    assert eq(ddf.corr(), df.corr())
+    assert eq(ddf.corr(min_periods=10), df.corr(min_periods=10))
+    assert ddf.corr()._name == ddf.corr()._name
+    assert ddf.corr(min_periods=10)._name != ddf.corr()._name
+
+    pytest.raises(NotImplementedError, lambda: ddf.corr(method='spearman'))
+
+    a = df.A
+    b = df.B
+    da = dd.from_pandas(a, npartitions=3)
+    db = dd.from_pandas(b, npartitions=4)
+    assert eq(da.corr(db), a.corr(b))
+    assert eq(da.corr(db, min_periods=10), a.corr(b, min_periods=10))
+    assert da.corr(db)._name == da.corr(db)._name
+    assert da.corr(db, min_periods=10)._name != da.corr(db)._name
+
+    pytest.raises(NotImplementedError, lambda: da.corr(db, method='spearman'))
+    pytest.raises(TypeError, lambda: da.corr(ddf))
+
 
 def test_apply_infer_columns():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
@@ -1490,6 +1575,10 @@ def test_from_imperative():
     assert s.compute().name == s.name
     assert list(s.map_partitions(len).compute()) == [1, 2, 3, 4]
 
+    df = dd.from_imperative(dfs, tm.makeTimeDataFrame(1))
+    assert df._known_dtype
+    assert list(df.columns) == ['A', 'B', 'C', 'D']
+
 
 def test_to_imperative():
     from dask.imperative import Value
