@@ -276,6 +276,8 @@ def test_from_bcolz():
     assert str(d.dtypes['a']) == 'category'
     assert list(d.x.compute(get=get_sync)) == [1, 2, 3]
     assert list(d.a.compute(get=get_sync)) == ['a', 'b', 'a']
+    L = list(d.index.compute(get=get_sync))
+    assert L == [0, 1, 2]
 
     d = dd.from_bcolz(t, chunksize=2, index='x')
     L = list(d.index.compute(get=get_sync))
@@ -399,6 +401,16 @@ def test_from_pandas_dataframe():
     assert len(ddf.divisions) == len(ddf.dask) + 1
     assert type(ddf.divisions[0]) == type(df.index[0])
     tm.assert_frame_equal(df, ddf.compute())
+    ddf = dd.from_pandas(df, chunksize=8)
+    msg = 'Exactly one of npartitions and chunksize must be specified.'
+    with tm.assertRaisesRegexp(ValueError, msg):
+        dd.from_pandas(df, npartitions=2, chunksize=2)
+    with tm.assertRaisesRegexp(ValueError, msg):
+        dd.from_pandas(df)
+    assert len(ddf.dask) == 3
+    assert len(ddf.divisions) == len(ddf.dask) + 1
+    assert type(ddf.divisions[0]) == type(df.index[0])
+    tm.assert_frame_equal(df, ddf.compute())
 
 
 def test_from_pandas_small():
@@ -409,6 +421,11 @@ def test_from_pandas_small():
         assert a.divisions[0] == 0
         assert a.divisions[-1] == 2
 
+        a = dd.from_pandas(df, chunksize=i)
+        assert len(a.compute()) == 3
+        assert a.divisions[0] == 0
+        assert a.divisions[-1] == 2
+
 
 def test_from_pandas_series():
     n = 20
@@ -420,12 +437,22 @@ def test_from_pandas_series():
     assert type(ds.divisions[0]) == type(s.index[0])
     tm.assert_series_equal(s, ds.compute())
 
+    ds = dd.from_pandas(s, chunksize=8)
+    assert len(ds.dask) == 3
+    assert len(ds.divisions) == len(ds.dask) + 1
+    assert type(ds.divisions[0]) == type(s.index[0])
+    tm.assert_series_equal(s, ds.compute())
+
 
 def test_from_pandas_non_sorted():
     df = pd.DataFrame({'x': [1, 2, 3]}, index=[3, 1, 2])
     ddf = dd.from_pandas(df, npartitions=2, sort=False)
     assert not ddf.known_divisions
     eq(df, ddf)
+    
+    ddf = dd.from_pandas(df, chunksize=2, sort=False)
+    assert not ddf.known_divisions
+    eq(df, ddf)
 
 
 def test_DataFrame_from_dask_array():
@@ -655,6 +682,7 @@ def test_read_hdf():
         df.to_hdf(fn, '/data', format='table')
         a = dd.read_hdf(fn, '/data', chunksize=2)
         assert a.npartitions == 2
+        assert a._known_dtype
 
         tm.assert_frame_equal(a.compute(), df)
 
@@ -902,7 +930,8 @@ def test_from_pandas_with_datetime_index():
                          parse_dates=['Date'])
         ddf = dd.from_pandas(df, 2)
         eq(df, ddf)
-
+        ddf = dd.from_pandas(df, chunksize=2)
+        eq(df, ddf)
 
 @pytest.mark.parametrize('encoding', ['utf-16', 'utf-16-le', 'utf-16-be'])
 def test_encoding_gh601(encoding):
@@ -991,3 +1020,13 @@ def test_read_csv_sep():
 
         assert (df.columns == ddf.columns).all()
         assert len(df) == len(ddf)
+
+
+def test_to_hdf_kwargs():
+    df = pd.DataFrame({'A': ['a', 'aaaa']})
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf.to_hdf('tst.h5', 'foo4', format='table', min_itemsize=4)
+
+    df2 = pd.read_hdf('tst.h5', 'foo4')
+
+    tm.assert_frame_equal(df, df2)
